@@ -87,12 +87,24 @@ var LuigiAPI = (function() {
         });
     };
 
+    LuigiAPI.prototype.getTaskStatusMessage = function(taskId, callback) {
+        return jsonRPC(this.urlRoot + "/get_task_status_message", {task_id: taskId}, function(response) {
+            callback(response.response);
+        });
+    };
+
     LuigiAPI.prototype.getRunningTaskList = function(callback) {
         return jsonRPC(this.urlRoot + "/task_list", {status: "RUNNING", upstream_status: "", search: searchTerm()}, function(response) {
             callback(flatten(response.response));
         });
     };
 
+    LuigiAPI.prototype.getBatchRunningTaskList = function(callback) {
+        return jsonRPC(this.urlRoot + "/task_list", {status: "BATCH_RUNNING", upstream_status: "", search: searchTerm()}, function(response) {
+            callback(flatten(response.response));
+        });
+    };
+
     LuigiAPI.prototype.getPendingTaskList = function(callback) {
         return jsonRPC(this.urlRoot + "/task_list", {status: "PENDING", upstream_status: "", search: searchTerm()}, function(response) {
             callback(flatten(response.response));
